'use strict'

const Debug = require('debug')
const split = require('split')

// Override debug's default coercion so buffers/errors become useful strings
Debug.coerce = function coerce (val) {
  if (val instanceof Error) return val.stack || val.message
  return val.toString()
}

let processes = []
let exitCode = 0

module.exports.killOnExit = killOnExit
module.exports.stderrNamespace = stderrNamespace

/**
 * Set opts.killAll to bring down all processes if any single process dies.
 */

function killOnExit (proc, opts) {
  processes.push(proc)

  opts = opts || {}
  const killAll = !!opts.killAll

  killOnExitSetup()

  proc.once('exit', code => exitCode = code ? 1 : exitCode)
  proc.once('error', () => exitCode = 1)
  proc.once('error', err => console.error(err))

  proc.once('error', onEnd)
  proc.once('close', onEnd)

  const cleanupAndExit = CleanupAndExit(proc)

  function onEnd () {
    processes = processes.filter(p => p.pid !== proc.pid)
    if (killAll) cleanupAndExit()
  }
}

function killAll () {
  if (killAll.called) return killAll.called
  killAll.called = new Promise((yeah, nah) => {
    processes.forEach(proc => {
      // wait for a bit for kill then terminate
      const timeout = setTimeout(() => {
        if (proc.connected) proc.kill('SIGTERM')
      }, 2000)

      proc.once('close', () => {
        clearTimeout(timeout)
        maybeDone()
      })
      proc.kill()
    })

    function maybeDone () {
      if (processes.length) return
      yeah()
    }
  })

  return killAll.called
}

function CleanupAndExit (proc) {
  return function cleanupAndExit () {
    if (CleanupAndExit.called) return
    CleanupAndExit.called = true
    if (proc !== process) {
      var args = proc.spawnargs || []
      args = args.slice(1) // remove first arg

      console.error(
        'Shutting down because process %s exited with code %s: %s %s',
        proc.pid, proc.exitCode, proc.spawnfile, args.join(' ')
      )
    }
    killAll().then(() => process.exit(exitCode || 0))
  }
}

function killOnExitSetup () {
  if (killOnExitSetup.called) return
  killOnExitSetup.called = true
  const cleanupAndExit = CleanupAndExit(process)
  process.on('SIGINT', cleanupAndExit)
  process.on('SIGTERM', cleanupAndExit)
  process.on('beforeExit', cleanupAndExit)
}

function stderrNamespace (namespace, proc, opts) {
  const logger = Debug(namespace)

  opts = opts || {}
  const invertStdio = !!opts.invertStdio
  const stdout = invertStdio ? 'stderr' : 'stdout'
  const stderr = invertStdio ? 'stdout' : 'stderr'

  proc[stdout]
  .pipe(process.stdout)

  proc[stderr]
  .pipe(split())
  .on('data', logger)

  killOnExit(proc, {killAll: true})
  return proc
}
