/**
 * In-memory config store.
 */

const yargs = require('yargs')
const path = require('path')
const configPath = require('application-config-path')('nsolid-console')
const fs = require('fs')
const debug = require('debug')('nsolid-console:config')
const mkdirp = require('mkdirp')
const merge = require('merge')

module.exports = getConfig

const DEFAULT_USAGE = `Start the N|Solid Console server
Usage: $0 [options]
`.trim()

const configDefaults = {
  hub: ['http://localhost:9000'],
  timeout: 15000,
  storage: configPath,
  duration: 15,
  interval: 5000,
  securityinterval: 1800000, // 30 minutes
  port: process.env.PORT || 3000,
  config: path.join(configPath, 'config'),
  thresholds: {},
  baseUrl: 'http://localhost:3000/',
  notificationEndpoint: {
    url: 'https://services.nodesource.io/notification'
  },
  licensekeyEndpoint: {
    url: 'https://services.nodesource.io/licenses'
  },
  securityEndpoint: {
    url: 'https://services.nodesource.io/vulnerabilities'
  },
  thresholdActionConfig: {
    profileDuration: 30 // in seconds
  },
  securityConfig: {
    enabled: true,
    auto: true,
    changed: false,
    recipients: []
  },
  license: {
    key: '',
    email: '',
    status: 'EMPTY'
  }
}

const configKeys = Object.keys(configDefaults)

function load (file) {
  const defaults = Object.assign({}, configDefaults)
  const configFile = file || path.join(configPath, 'config')

  try {
    const contents = fs.readFileSync(configFile, 'utf8')
    const conf = Object.assign(defaults, JSON.parse(contents))

    // TODO: check license key here

    // propagate the file that this config was loaded from, or
    // utilize the default config location
    conf.config = configFile
    debug('loaded config: %s', configFile)

    return conf
  } catch (e) {
    return Object.assign(defaults, {
      config: configFile
    })
  }
}

function save (config, fn) {
  const out = {}

  configKeys.forEach((key) => {
    out[key] = config[key] || config.rc[key] || configDefaults[key]
  })

  const configFile = config.rc.config
  if (!configFile) {
    throw new Error('not implemented')
  }

  const configString = JSON.stringify(out, null, 2)

  debug('save config: %s', configFile)
  debug(configString)

  const configDir = path.dirname(configFile)

  mkdirp(configDir, (dirError) => {
    if (dirError) {
      return fn(dirError)
    }

    fs.writeFile(configFile, configString, (e) => {
      if (e) {
        debug('error saving config file')
        debug(e.stack)
        return fn(e)
      }

      fn()
    })
  })
}

function getConfig (args, usage) {
  const defaults = Object.assign({}, configDefaults)
  const defaultConfigFile = path.join(configPath, 'config')

  const argv = yargs
  .usage(usage || DEFAULT_USAGE)
  .describe('config', 'Configuration file location')
  .default('config', defaultConfigFile)
  .describe('duration', 'Maximum time to retain graph data in minutes')
  .default('duration', defaults.duration)
  .alias('d', 'duration')
  .nargs('duration', 1)
  .describe('interface', 'Networking interface HTTP server will listen on')
  .default('interface', '0.0.0.0')
  .nargs('interface', 1)
  .describe('interval', 'Frequency to poll N|Solid Hub in ms')
  .default('interval', defaults.interval)
  .alias('i', 'interval')
  .nargs('interval', 1)
  .describe('port', 'Port HTTP server will listen on')
  .default('port', defaults.port)
  .describe('securityinterval', 'Delay in ms between vulnerability checks on all packages')
  .nargs('securityinterval', 1)
  .default('securityinterval', defaults.securityinterval)
  .alias('p', 'port')
  .nargs('port', 1)
  .alias('r', 'record')
  .describe('storage', 'Location on disk for storing snapshots & profiles')
  .alias('s', 'storage')
  .default('storage', defaults.storage)
  .nargs('storage', 1)
  .boolean('flush')
  .describe('flush', 'Flush persisted data and configuration on startup.')
  .default('flush', false)
  .option('hub', { array: true })
  .describe('hub', 'Space separated list of N|Solid Hub(s) (e.g., --hub=http://localhost:9000 http://domain.tld:9000)')
  .default('hub', defaults.hub)
  .describe('timeout', 'Maximum amount of time that a request to the N|Solid Hub can take in ms')
  .default('timeout', defaults.timeout)
  .nargs('timeout', 1)
  .help('help')
  .alias('help', 'h')
  .parse(args || process.argv)

  // ensure paths are resolved against cwd
  if (argv.storage) {
    argv.storage = path.resolve(process.cwd(), argv.storage)
  }

  if (argv.record) {
    argv.record = path.resolve(process.cwd(), argv.record)
  }

  const conf = argv.flush ? Object.assign(defaults, { config: argv.config }) : load(argv.config)

  if (!argv.config) {
    argv.config = conf.config
  }

  argv.rc = merge(conf, argv.rc)

  return argv
}

getConfig.load = load
getConfig.save = save

getConfig.defaults = Object.assign({}, configDefaults)
