'use strict'

const async = require('async')
const url = require('url')
const path = require('path')
const debug = require('debug')('nsolid-console:ThresholdAction')
const computeAppId = require('../../compute-app-id')
const doSnapshot = require('../actions/snapshot')
const doProfile = require('../actions/profile')
const doNotify = require('../actions/notify')

class ThresholdAction {
  constructor (metric, value, cache, state, thresholdConfig) {
    this.metric = metric
    this.state = state
    this.cache = cache
    this.thresholdConfig = thresholdConfig
    this.actions = {
      snapshot: function performSnapshot (cache, appConfig, actionConfig, proc, done) {
        debug('snapshot on process %s (%s)', proc.id, proc.app)
        doSnapshot(cache, appConfig, proc, (e, r) => {
          if (e) {
            return done(e)
          }

          const heapUrl = url.resolve(appConfig.rc.baseUrl, '/' + path.join(
            'app',
            computeAppId(r.app),
            'heap',
            r.id,
            r.time + ''
          ))

          done(null, heapUrl, r)
        })
      },
      profile: function performProfile (cache, appConfig, actionConfig, proc, done) {
        const duration = appConfig.rc.thresholdActionConfig.profileDuration * 1000
        debug('profile on process %s (%s) for %s', proc.id, proc.app, duration)
        doProfile(cache, appConfig, proc, duration, (e, r) => {
          if (e) {
            return done(e)
          }

          const profileUrl = url.resolve(appConfig.rc.baseUrl, '/' + path.join(
            'app',
            computeAppId(r.app),
            'profile',
            r.id,
            r.time + '',
            'treemap'
          ))

          done(null, profileUrl, r)
        })
      },
      notify: function performNotify (cache, appConfig, results, actionConfig, proc, done) {
        const payload = {
          type: 'threshold',
          method: 'email',
          data: {
            metric: metric,
            value: value,
            threshold: thresholdConfig.threshold,
            recipients: actionConfig.recipients,
            delay: thresholdConfig.delay,
            delayUnits: thresholdConfig.delayUnits,
            process: proc,
            artifacts: results
          }
        }

        debug('notify on process %s (%s)', proc.id, proc.app)

        doNotify(appConfig, payload, done)
      }
    }
  }

  getActions () {
    if (!this.thresholdConfig) {
      return false
    }

    const actions = this.thresholdConfig.actions
    if (!actions) {
      return false
    }

    return actions
  }

  execute (config, next) {
    const configuredActions = this.getActions()
    const actions = this.actions
    const metric = this.metric
    const cache = this.cache
    const state = this.state

    const appid = state.app
    const processid = state.id

    if (!configuredActions) {
      return finish()
    }

    let willNotify = false

    const actionNames = Object.keys(configuredActions).filter((key) => {
      if (key === 'notify') {
        willNotify = !!configuredActions.notify.enabled
        return false
      }

      return true
    })

    async.mapSeries(actionNames, processAction, mapComplete)

    function processAction (actionName, done) {
      const action = actions[actionName]
      const actionConfig = configuredActions[actionName]

      if (!action || !actionConfig || !actionConfig.enabled) {
        return done()
      }

      action(cache, config, actionConfig, state, (e, resourceUrl, result) => {
        if (e) {
          return done(null, {
            error: {
              message: e.message,
              code: e.code
            }
          })
        }

        if (!result && !resourceUrl) {
          return done()
        }

        done(null, {
          type: actionName,
          result: result,
          url: resourceUrl
        })
      })
    }

    function notify (results, fn) {
      actions.notify(cache, config, results, configuredActions.notify, state, (e, r) => {
        const artifact = {
          type: 'notify'
        }

        if (e) {
          artifact.error = {
            message: e.message,
            code: e.code
          }
        } else {
          artifact.result = r
        }

        results.push(artifact)

        finish(e, results)
      })
    }

    function mapComplete (err, results) {
      if (willNotify && results) {
        return notify(results.filter(Boolean), finish)
      }

      finish(err, results)
    }

    function finish (e, r) {
      const proc = cache.getProcess(appid, processid)

      // if the process crashed, avoid exploding
      if (!proc) {
        return next()
      }

      if (!proc.thresholds) {
        proc.thresholds = {}
      }
      proc.thresholds[metric] = { snoozeStart: Date.now() }
      next(e, r)
    }
  }
}

module.exports = ThresholdAction
