/*

Compute the union between process metrics and threshold action configuration, which may
result in performing a configured action (notify, snapshot, profile).

*/

'use strict'

const ThresholdAction = require('./threshold/action')

const utils = require('../utils')
const toMilliseconds = utils.toMilliseconds
const timerElapsed = utils.timerElapsed

const debug = require('debug')('nsolid-console:thresholds')
module.exports = processThresholds

const METRICS = ['cpu', 'heap']
const GB = 1024 * 1024 * 1024
const METRIC_MAPPING = {
  cpu (obj) {
    return obj.cpu / 100
  },
  heap (obj) {
    return obj.memory.used / (1.5 * GB)
  }
}

/*
 * Mutates incoming process, if the threshold is crossed, to include threshold crossing
 * time and value. If a process has gone over the threshold for the configured `delay`
 * and is not currently in the `snooze` period, then an action will be added to the queue.
 *
 * @param {state}               object massaged process as defined in build-response
 * @param {cache}               object instance of a cache object inside of a Cache.collectPollingResult block
 * @param {config}              object pre-filtered threshold configuration for this process' app
 * @param {rebuildActionQueue}  bool   override previous state in the case this is a fresh reboot
 */
function processThresholds (state, cache, config, rebuildActionQueue) {
  if (!config) {
    return
  }

  const appid = state.app
  const processid = state.id
  const previousCache = cache.previousCache
  const previousState = previousCache ? previousCache.getProcess(appid, processid) : null
  const currentThresholds = state.thresholds || {}
  const newThresholdState = {}

  const results = METRICS.map((metricName) => {
    const metricConfig = config[metricName]
    if (!metricConfig || !metricConfig.enabled) {
      return
    }

    const newState = newThresholdState[metricName] = {}
    const currentState = currentThresholds[metricName]

    if (rebuildActionQueue && currentState.snoozeStart === Number.MAX_VALUE) {
      newState.snoozeStart = Number.MAX_VALUE

      const action = new ThresholdAction(
        metricName,
        currentState.value,
        cache,
        state,
        metricConfig
      )

      cache.addThresholdAction(action)

      return action
    }

    const snoozeMs = toMilliseconds(metricConfig.snooze, metricConfig.snoozeUnits)
    const delayMs = toMilliseconds(metricConfig.delay, metricConfig.delayUnits)

    // compare the current state with the configured threshold
    const currentMetricValue = newState.value = METRIC_MAPPING[metricName](state)

    let previousThresholdState = {}

    // this only happens on the top level cache i.e. inside of Cache#fromJSON
    if (previousState && previousState.thresholds && previousState.thresholds[metricName]) {
      previousThresholdState = previousState.thresholds[metricName]
    }

    const snoozeStart = previousThresholdState.snoozeStart
    const snoozeElapsed = !snoozeStart || timerElapsed(snoozeStart, snoozeMs)

    // if this metric is not crossing the threshold there is nothing left to do
    const crossing = currentMetricValue > metricConfig.threshold
    newState.crossing = crossing

    if (!snoozeElapsed) {
      newState.snoozeStart = snoozeStart
      return debug('snoozing', Date.now() - snoozeStart, snoozeMs, processid)
    }

    if (!crossing) {
      return
    }

    // if this is the first cycle of collection for this particular process
    // then add delay start and be on our way
    if (!Object.keys(previousThresholdState).length) {
      newState.delayStart = Date.now()
      return
    }

    // forward previous state if delayStart was already set
    const delayStart = previousThresholdState.delayStart
    if (delayStart) {
      if (!timerElapsed(delayStart, delayMs)) {
        newState.delayStart = delayStart
        return debug('delay', Date.now() - delayStart, delayMs)
      } else {
        // snooze the process for an infinite amount of time to avoid
        // jamming a bunch of identical actions into the action queue
        newState.snoozeStart = Number.MAX_VALUE

        debug('add new threshold action for %s', metricName)
        const action = new ThresholdAction(
          metricName,
          currentMetricValue,

          // generally use the previous cache as the local cache gets merged back up immediately,
          // where as this action will take atleast 1 tick to complete
          previousCache,
          state,
          metricConfig
        )

        cache.addThresholdAction(action)
        return action
      }
    }

    // put this process into The Danger Zone™
    newState.delayStart = Date.now()
  }).filter(Boolean)

  if (Object.keys(newThresholdState).length) {
    state.thresholds = newThresholdState
  }

  return results
}
