'use strict'

const test = require('tape')
const Unread = require('../unread')

class MockStorage {
  constructor () {
    this.data = {}
  }

  setItem (key, value) {
    // insist value is string to match webstorage
    this.data[key] = String(value)
    return
  }

  getItem (key) {
    return this.data[key]
  }
}

test('unread – health checks', t => {
  t.equal(typeof Unread, 'function', 'Unread is a function')
  t.ok(new Unread(new MockStorage()), 'new Unread can be instantiated')
  t.end()
})

test('unread – reading items', t => {
  const unread = new Unread(new MockStorage())
  t.notOk(unread.isRead('1'), 'isRead is false for unread item 1')
  unread.read('1')
  unread.read('1')
  unread.read('2')
  t.ok(unread.isRead('1'), 'isRead is true for read item 1')
  t.ok(unread.isRead('2'), 'isRead is true for read item 2')
  unread.unread('1')
  unread.unread('3') // should do nothing
  t.notOk(unread.isRead('1'), 'isRead is false for unread item 1')
  unread.unread('1') // also do nothing
  t.notOk(unread.isRead('1'), 'isRead is false for unread item 1')
  t.ok(unread.isRead('2'), 'isRead is true for read item 2')
  t.notOk(unread.isRead('3'), 'isRead is false for invalidly unread item 3')
  t.end()
})

test('unread - bad inputs', t => {
  const unread = new Unread(new MockStorage())
  t.notOk(unread.isRead(), 'isRead falsey for invalid input')
  t.notOk(unread.read(), 'read falsey for invalid input')
  t.notOk(unread.unread(), 'read falsey for invalid input')
  t.notOk(unread.toggleRead(), 'toggleRead falsey for invalid input')
  t.notOk(unread.readMany(), 'falsey for invalid input')
  t.end()
})

test('unread – toggleRead', t => {
  const unread = new Unread(new MockStorage())
  t.ok(unread.toggleRead('1'), 'toggleRead is true for toggled to read item 1')
  t.ok(unread.isRead('1'), 'isRead is true for read item 1')
  t.notOk(unread.toggleRead('1'), 'toggleRead is false for toggled to unread item 1')
  t.notOk(unread.isRead('1'), 'isRead is false for unread item 1')
  t.ok(unread.toggleRead('1'), 'toggleRead is true for toggled to read item 1')
  t.end()
})

test('unread – readMany', t => {
  const unread = new Unread(new MockStorage())
  unread.readMany(['1', '2', '3'])
  t.ok(unread.isRead('1'), 'isRead is true for read item 1')
  t.ok(unread.isRead('2'), 'isRead is true for read item 2')
  t.ok(unread.isRead('3'), 'isRead is true for read item 3')
  t.notOk(unread.isRead('4'), 'isRead is false for unread item 4')
  t.end()
})

test('unread – bulkReadCheck gives cached reader', t => {
  const unread = new Unread(new MockStorage())
  unread.readMany(['1', '2', '3'])
  const isRead = unread.bulkReadCheck()
  t.ok(isRead('1'), 'bulkReadCheck is true for read item 1')
  t.ok(isRead('2'), 'bulkReadCheck is true for read item 2')
  t.ok(isRead('3'), 'bulkReadCheck is true for read item 3')
  t.notOk(isRead('4'), 'bulkReadCheck is false for unread item 4')
  unread.unread('1')
  t.ok(isRead('1'), 'bulkReadCheck is true for read item 1 as it is cached')
  t.notOk(unread.isRead('1'), 'isRead is true for real read item 1')
  t.end()
})
